""" Copyright 2012, 2013 UW Information Technology, University of Washington

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from django.test import TestCase
from django.conf import settings
from django.test.client import Client
from django.core.files import File
from spotseeker_server.models import Spot, SpotImage
import os
from django.test.utils import override_settings
from mock import patch
from spotseeker_server import models

TEST_ROOT = os.path.abspath(os.path.dirname(__file__))


@override_settings(SPOTSEEKER_AUTH_MODULE='spotseeker_server.auth.all_ok')
@override_settings(SPOTSEEKER_AUTH_ADMINS=('demo_user',))
@override_settings(MEDIA_ROOT=os.path.join(TEST_ROOT, '..', 'resources'))
class SpotImageDELETETest(TestCase):
    """ Tests DELETE of a SpotImage at /api/v1/spot/<spot id>/image/<image id>.
    """
    def setUp(self):
        spot = Spot.objects.create(name="This is to test DELETEing images")
        spot.save()
        self.spot = spot

        self.url = '/api/v1/spot/{0}'.format(self.spot.pk)
        self.url = self.url

        # GIF
        f = open("%s/../resources/test_gif.gif" % TEST_ROOT, 'rb')
        gif = self.spot.spotimage_set.create(
            description="This is the GIF test",
            image=File(f))
        f.close()

        self.gif = gif
        self.gif_url = "%s/image/%s" % (self.url, self.gif.pk)

        # JPEG
        f = open("%s/../resources/test_jpeg.jpg" % TEST_ROOT, 'rb')
        jpeg = self.spot.spotimage_set.create(
            description="This is the JPEG test",
            image=File(f))
        f.close()

        self.jpeg = jpeg
        self.jpeg_url = "%s/image/%s" % (self.url, self.jpeg.pk)

        # PNG
        f = open("%s/../resources/test_png.png" % TEST_ROOT, 'rb')
        png = self.spot.spotimage_set.create(
            description="This is the PNG test",
            image=File(f))
        f.close()

        self.png = png
        self.png_url = "%s/image/%s" % (self.url, self.png.pk)

    def test_bad_url(self):
        c = Client()
        bad_url = "%s/image/aa" % self.url
        response = c.delete(bad_url)
        self.assertEqual(response.status_code, 404)

    def test_wrong_spot_id(self):
        c = Client()
        spot = Spot.objects.create(name="This is the wrong spot")

        f = open("%s/../resources/test_png.png" % TEST_ROOT, 'rb')
        png = self.spot.spotimage_set.create(
            description="This is another PNG", image=File(f))
        f.close()

        response = \
            c.delete("/api/v1/spot/{0}/image/{1}".format(spot.pk, png.pk))
        self.assertEqual(response.status_code, 404)

    def test_invalid_id_too_high(self):
        c = Client()

        # GIF
        test_gif_id = self.gif.pk + 10000
        test_url = "/api/v1/spot/%s/image/%s" % (self.url, test_gif_id)
        response = c.delete(test_url)
        self.assertEqual(response.status_code, 404)

    def test_actual_delete_with_etag(self):
        c = Client()

        # GIF
        response = c.get(self.gif_url)
        etag = response["ETag"]

        response = c.delete(self.gif_url, If_Match=etag)

        self.assertEqual(response.status_code, 200)

        response = c.get(self.gif_url)
        self.assertEqual(response.status_code, 404)

        response = c.delete(self.gif_url)
        self.assertEqual(response.status_code, 404)

        self.assertEqual(os.path.isfile(self.gif.image.path), False)

        try:
            test_gif = SpotImage.objects.get(pk=self.gif.pk)
        except Exception as e:
            test_gif = None

        self.assertIsNone(test_gif)

        # JPEG
        response = c.get(self.jpeg_url)
        etag = response["ETag"]

        response = c.delete(self.jpeg_url, If_Match=etag)

        self.assertEqual(response.status_code, 200)

        response = c.get(self.jpeg_url)
        self.assertEqual(response.status_code, 404)

        response = c.delete(self.jpeg_url)
        self.assertEqual(response.status_code, 404)

        self.assertEqual(os.path.isfile(self.jpeg.image.path), False)

        try:
            test_jpeg = SpotImage.objects.get(pk=self.jpeg.pk)
        except Exception as e:
            test_jpeg = None

        self.assertIsNone(test_jpeg)

        # PNG
        response = c.get(self.png_url)
        etag = response["ETag"]

        response = c.delete(self.png_url, If_Match=etag)

        self.assertEqual(response.status_code, 200)

        response = c.get(self.png_url)
        self.assertEqual(response.status_code, 404)

        response = c.delete(self.png_url)
        self.assertEqual(response.status_code, 404)

        self.assertEqual(os.path.isfile(self.png.image.path), False)

        try:
            test_png = SpotImage.objects.get(pk=self.png.pk)
        except Exception as e:
            test_png = None

        self.assertIsNone(test_png)

    def test_actual_delete_no_etag(self):
        c = Client()

        # GIF
        response = c.delete(self.gif_url)
        self.assertEqual(response.status_code, 400)

        response = c.get(self.gif_url)
        self.assertEqual(response.status_code, 200)

        # JPEG
        response = c.delete(self.jpeg_url)
        self.assertEqual(response.status_code, 400)

        response = c.get(self.jpeg_url)
        self.assertEqual(response.status_code, 200)

        # PNG
        response = c.delete(self.png_url)
        self.assertEqual(response.status_code, 400)

        response = c.get(self.png_url)
        self.assertEqual(response.status_code, 200)

    def test_actual_delete_expired_etag(self):
        c = Client()

        # GIF
        response = c.get(self.gif_url)
        etag = response["ETag"]

        intermediate_img = SpotImage.objects.get(pk=self.gif.pk)
        intermediate_img.name = "This interferes w/ the DELETE"
        intermediate_img.save()

        response = c.delete(self.gif_url, If_Match=etag)
        self.assertEqual(response.status_code, 409)

        response = c.get(self.gif_url)
        self.assertEqual(response.status_code, 200)

        # JPEG
        response = c.get(self.jpeg_url)
        etag = response["ETag"]

        intermediate_img = SpotImage.objects.get(pk=self.jpeg.pk)
        intermediate_img.name = "This interferes w/ the DELETE"
        intermediate_img.save()

        response = c.delete(self.jpeg_url, If_Match=etag)
        self.assertEqual(response.status_code, 409)

        response = c.get(self.jpeg_url)
        self.assertEqual(response.status_code, 200)

        # PNG
        response = c.get(self.png_url)
        etag = response["ETag"]

        intermediate_img = SpotImage.objects.get(pk=self.png.pk)
        intermediate_img.name = "This interferes w/ the DELETE"
        intermediate_img.save()

        response = c.delete(self.png_url, If_Match=etag)
        self.assertEqual(response.status_code, 409)

        response = c.get(self.png_url)
        self.assertEqual(response.status_code, 200)
